#include <galprop/SourceDistribution.h>
#include "galpropy.hpp"

class PySourceDistributionVirtual : public SourceDistributionVirtual {
public:
    using SourceDistributionVirtual::SourceDistributionVirtual;
    const Galprop* galprop_ptr(void) const override {
        PYBIND11_OVERRIDE(const Galprop*, SourceDistributionVirtual,
                          galprop_ptr,);
    }
    double density_2d(double r, double z) override {
        PYBIND11_OVERRIDE(double, SourceDistributionVirtual, density_2d, r, z);
    }
    double density_3d(double x, double y, double z) override {
        PYBIND11_OVERRIDE_PURE(double, SourceDistributionVirtual,
                               density_3d, x, y, z);
    }
    std::valarray<double> distribution_2d(
        const std::valarray<double>& r,
        const std::valarray<double>& z) override {
        PYBIND11_OVERRIDE(std::valarray<double>, SourceDistributionVirtual,
                          distribution_2d, r, z);
    }
    std::valarray<double> distribution_3d(
        const std::valarray<double>& x,
        const std::valarray<double>& y,
        const std::valarray<double>& z) override {
        PYBIND11_OVERRIDE(std::valarray<double>, SourceDistributionVirtual,
                          distribution_3d, x, y, z);
    }
};


class PySrcDistParameterized : public SrcDistParameterized {
public:
    using SrcDistParameterized::SrcDistParameterized;
    double density_3d(double x, double y, double z) override {
        PYBIND11_OVERRIDE(double, SrcDistParameterized,
                          density_3d, x, y, z);
    }
    bool is_electron(void) const override {
        PYBIND11_OVERRIDE_PURE(bool, SrcDistParameterized, is_electron,);
    }
};

class PyEleSrcDistParameterized : public EleSrcDistParameterized {
public:
    using EleSrcDistParameterized::EleSrcDistParameterized;
    bool is_electron(void) const override {
        PYBIND11_OVERRIDE(bool, EleSrcDistParameterized, is_electron,);
    }
};

class PyNucSrcDistParameterized : public NucSrcDistParameterized {
public:
    using NucSrcDistParameterized::NucSrcDistParameterized;
    bool is_electron(void) const override {
        PYBIND11_OVERRIDE(bool, NucSrcDistParameterized, is_electron,);
    }
};

void bind_SourceDistribution(py::module_ &m)
{
    py::class_<SourceDistributionVirtual, PySourceDistributionVirtual,
               std::shared_ptr<SourceDistributionVirtual>>(
        m, "SourceDistributionVirtual")
        .def(py::init_alias<>())
        .def("galprop_ptr", &SourceDistributionVirtual::galprop_ptr)
        .def("density_2d", &SourceDistributionVirtual::density_2d)
        .def("density_3d", &SourceDistributionVirtual::density_3d)
        .def("distribution_2d", &SourceDistributionVirtual::distribution_2d)
        .def("distribution_3d", &SourceDistributionVirtual::distribution_3d)
        .def_readwrite("pG", &SourceDistributionVirtual::pG);

    // TODO use trampoline template
    py::class_<SrcDistParameterized, PySrcDistParameterized,
               SourceDistributionVirtual,
               std::shared_ptr<SrcDistParameterized>>(
        m, "SrcDistParameterized")
        .def(py::init_alias<>())
        .def("is_electron", &SrcDistParameterized::is_electron)
        .def("density_3d", &SrcDistParameterized::density_3d);

    py::class_<EleSrcDistParameterized, PyEleSrcDistParameterized,
               SrcDistParameterized, std::shared_ptr<EleSrcDistParameterized>>(
        m, "EleSrcDistParameterized")
        .def(py::init_alias<>())
        .def("is_electron", &EleSrcDistParameterized::is_electron);

    py::class_<NucSrcDistParameterized, PyNucSrcDistParameterized,
               SrcDistParameterized, std::shared_ptr<NucSrcDistParameterized>>(
        m, "NucSrcDistParameterized")
        .def(py::init_alias<>())
        .def("is_electron", &NucSrcDistParameterized::is_electron);
}
