#include <galprop/SourceSpectrum.h>
#include "galpropy.hpp"

class PySourceSpectrumVirtual : public SourceSpectrumVirtual {
public:
    using SourceSpectrumVirtual::SourceSpectrumVirtual;
    std::valarray<double> spectrum(const Particle& p) const override {
        PYBIND11_OVERRIDE_PURE(std::valarray<double>, SourceSpectrumVirtual,
                               spectrum, p);
    }
};

void bind_SourceSpectrum(py::module_ &m)
{
    using srcseg = SrcSpecSBPL::SmoothlyBrokenSegment;
    using vecseg = std::vector<srcseg>;
    py::bind_vector<vecseg>(m, "VecSmoothlyBrokenSegment");
    py::implicitly_convertible<py::list, vecseg>();
    py::implicitly_convertible<py::tuple, vecseg>();

    py::class_<SourceSpectrumVirtual, PySourceSpectrumVirtual,
               std::shared_ptr<SourceSpectrumVirtual>>(
        m, "SourceSpectrumVirtual")
        .def(py::init_alias<>());

    py::class_<SrcSpecSBPL, SourceSpectrumVirtual,
               std::shared_ptr<SrcSpecSBPL>> src(m, "SrcSpecSBPL");
    src.def(py::init<>())
        .def("set_norm_ekn_GeV", &SrcSpecSBPL::set_norm_ekn_GeV)
        .def("set_norm_rig_GV", &SrcSpecSBPL::set_norm_rig_GV)
        .def("spectrum", &SrcSpecSBPL::spectrum)
        .def_readwrite("abundance", &SrcSpecSBPL::abundance)
        .def_readwrite("norm_ekn", &SrcSpecSBPL::norm_ekn)
        .def_readwrite("norm_rig", &SrcSpecSBPL::norm_rig)
        .def_readwrite("cutoff_rig", &SrcSpecSBPL::cutoff_rig)
        .def_readwrite("segments", &SrcSpecSBPL::segments);

    py::class_<srcseg>(src, "SmoothlyBrokenSegment")
        .def(py::init<>())
        .def(py::init<double, double, double>(),
             "gamma"_a, "rig"_a=HUGE_VAL, "s"_a=0)
        .def(py::init([](const py::tuple& args) {
                std::size_t arg_len = py::len(args);
                if (arg_len < 1 || arg_len > 3) {
                    throw py::value_error("tuple length incorrect");
                }
                double gamma = args[0].cast<double>(), rig = HUGE_VAL, s = 0;
                if (arg_len > 1) {
                    rig = args[1].cast<double>();
                    if (arg_len > 2) {
                        s = args[2].cast<double>();
                    }
                }
                return srcseg {gamma, rig, s};
            }))
        .def_readwrite("gamma", &srcseg::gamma)
        .def_readwrite("rig", &srcseg::rig)
        .def_readwrite("s", &srcseg::s);

    py::implicitly_convertible<py::tuple, srcseg>();
}
